program Desig_test
  ! Program for testing designature
  ! Adapted from Ettore's Moerevest code
  !
  ! Inputs:
  !
  ! Outputs:
  !
  ! Jason Chang
  ! February 2016


  !=========================================
  ! standard declarations
  !=========================================
  use sep
  use resample
  use fourier
  implicit none

  !=========================================
  ! variable type
  !=========================================
  ! program parameters
  integer                             :: ii
  integer                             :: ftlen
  real                                :: eps,sign1
  real,allocatable,dimension(:)       :: output

  ! input file parameters
  integer                             :: n1,n2,n3
  real                                :: o1,o2,o3,d1,d2,d3
  real                                :: epsd
  real,allocatable,dimension(:)       :: datain_t,in_amp
  complex,allocatable,dimension(:,:)  :: datain_f

  ! filter parameters
  integer                             :: n1f
  integer                             :: padf
  real                                :: o1f,d1f
  real                                :: epsf
  real,allocatable,dimension(:)       :: datair_t,ir_amp
  complex,allocatable,dimension(:,:)  :: datair_f


  !=========================================
  ! declare parameters
  !=========================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)

  ! filter parameters
  if (exist_file('filter_in')) then
    call from_aux('filter_in','n1',n1f,1 )
    call from_aux('filter_in','o1',o1f,0.)
    call from_aux('filter_in','d1',d1f,1.)
  else
    call seperr('Need file containing trace to deconvolve with')
  end if

  ! epsilon parameter
  call from_param('eps',eps,3.5e-3)

  if (exist_file('filter_out')) then
    !call to_history('n1',n1,'filter_out')
    call to_history('n1',n1f,'filter_out')
    call to_history('o1',o1f,'filter_out')
    call to_history('d1',d1,'filter_out')
    call to_history('n2',1 ,'filter_out')
    call to_history('o2',0.,'filter_out')
    call to_history('d2',1.,'filter_out')
  end if


  !=========================================
  ! initializations
  !=========================================
  ! allocations
  allocate(datair_t(n1f)); datair_t=0.
  allocate(datain_t(n1)) ; datain_t=0.

  ! resample the filter coefficients
  call sreed('filter_in',datair_t,4*n1f)
  call filt_resamp(d1f,d1,datair_t,n1f,padf)

  ! more allocations
  ftlen = max(padf,n1)
  sign1 = 1.
  allocate(datair_f(ftlen,1)); datair_f=0.
  allocate(datain_f(ftlen,1)); datain_f=0.
  allocate(ir_amp(ftlen))
  allocate(in_amp(ftlen))
  allocate(output(ftlen))

  ! initializations
  call ft_init(ftlen,1)
  

  !=========================================
  ! kernel
  !=========================================
  ! shift filter
  datair_t = cshift(datair_t,shift=padf/4)
  if (exist_file('filter_out')) then
    call srite('filter_out',datair_t,4*padf)
  end if

  ! change to instrument response to complex
  datair_f(1:padf/2,1) = cmplx(datair_t(1:padf/2),0.)
  datair_f(ftlen-padf/2:ftlen,1) = cmplx(datair_t(padf/2+1:padf),0.)

  ! call fft
  call ft1axis(.false.,sign1,datair_f)

  ! calculate scales for instrument response
  ir_amp   = cabs(datair_f(:,1))
  epsf     = eps*(sum(ir_amp))/ftlen
  datair_f(:,1) = datair_f(:,1)/(ir_amp+epsf)

  ! loop over traces
  do ii = 1,n2*n3
    ! read in data
    call sreed('in',datain_t,4*n1)

    ! perform fft
    datain_f(1:n1,1) = cmplx(datain_t,0.)
    call ft1axis(.false.,sign1,datain_f)

    ! calculate scales for data
    in_amp = cabs(datain_f(:,1))
    epsd   = eps*(sum(in_amp))/ftlen
    datain_f(:,1) = datain_f(:,1)/(in_amp+epsd)

    ! deconvolve
    datain_f(:,1) = (1./sqrt(float(ftlen)))*datain_f(:,1)*conjg(datair_f(:,1))*(in_amp/(ir_amp+epsd))

    ! perform ifft
    call ft1axis(.true.,sign1,datain_f)
    datain_t = real(datain_f(:,1))

    ! write out data
    call srite('out',datain_t,4*n1)
  end do


  !=========================================
  ! clean up
  !=========================================
  deallocate(datair_t,datain_t)
  deallocate(datair_f,datain_f)
  deallocate(ir_amp,in_amp)
  call sep_close()
  call ft_destroy()
  call exit(0)

end program
