program Debubble_tracebytrace
  ! Program for removing bubble from data
  ! Based on GEE's pef and helicon modules
  ! Trace-by-trace operation
  !
  ! Inputs:
  !   1) history file of data
  !   2) nw: length of wavelet window
  !   3) fw: starting point of wavelet window
  !   4) niter: number of iterations for pef
  !   5) nlag: length of filter
  !   6) ngap: length of gap
  !
  ! Outputs:
  !   1) history file of de-bubbled data
  !
  ! Bob Clapp
  ! February 2016


  !=================================
  ! standard declarations
  !=================================
  use sep
  use pef
  use helicon
  implicit none


  !=================================
  ! declare variables
  !=================================
  ! program variables
  integer                       :: ii,stat
  integer                       :: nlag,niter,ngap

  ! input traces
  integer                       :: n1,n2,n123
  real,dimension(:,:),pointer   :: in2d,out2d
  real, dimension(:),pointer    :: input,output,inw

  ! wavelet variables
  integer                       :: nw,fw
  type(filter)                  :: aa


  !=================================
  ! open I/O
  !=================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1)
  call from_history('n2',n2,1)

  ! wavelet window parameters
  call from_param('nw',nw,n1)
  call from_param('fw',fw,0)
  call from_param("niter",niter,100)
  call from_param("nlag",nlag,25)
  call from_param("ngap",ngap,50)


  !=================================
  ! initialization
  !=================================
  ! set variables
  n123=n1*n2

  ! allocate
  allocate(in2d(n1,n2),out2d(n1,n2),input(n1),output(n1))
  call allocatehelix(aa,nlag);
  allocate(aa%mis(n123))
  allocate(inw(nw))


  !=================================
  ! kernel
  !=================================
  ! read in data
  call sep_read(in2d)

  ! populate lags for filter
  ! essentially a gap deconvolution?
  do ii=1,nlag
    aa%lag(ii)=ii+ngap
  end do

  ! loop over traces
  do ii=1,n2
    ! set input trace
    input=in2d(:,ii)

    ! window used for estimating ped
    inw=in2d(fw+1:fw+nw+(ii-1)*(n2))
    call find_pef(inw,aa,niter)

    ! convolve filter with data
    call helicon_init(aa)
    stat=helicon_lop(.false.,.false.,input,output)
    out2d(:,ii)=output
  end do

  ! write out
  call sep_write(out2d)


  !=================================
  ! clean up
  !=================================
  call sep_close()
  call deallocatehelix(aa)
  deallocate(in2d,out2d,input,output,inw)
  call exit(0)

end program
