!! Yi Shen (Aug, 2015)
!!.. use lloyd to pick the reference pair (Q, V)
!!   nrf: call from param; the maximum reference number
!!   qscale: call from param: the scale for Q to balance Q and V (e.g. Q=1-4, V=1000-4000, qscale= 1000)
!!   nstart: call from param: the initial reference picking number for iref
!!   Output: iregion: out put the refence number (1, 2 or 3) for each location; iref: the reference number for each depth. sjould be smaller than nref
module bwi_ref_mod
use sep3d_struct_mod
use base_lloyd_mod 
use sep
  implicit none
  integer, private :: nrf, nx, ny
  real, private	   :: qscale
  integer,dimension(:), pointer, private :: nstart
contains

!=====================================================================
!
  integer function bwi_ref_init(nrf_in, nx_in, ny_in, qscale_in, nstart_in ) result(stat)
    integer :: nrf_in
    integer :: nx_in
    integer :: ny_in
    real    :: qscale_in
    integer,dimension(:), target:: nstart_in

    nrf  = nrf_in
    nx   = nx_in
    ny   = ny_in
    qscale = qscale_in
    nstart => nstart_in
    stat  = 0
  end function
!======================================================================
  integer function bwi_ref_clos() result(stat)
    nrf   = 0
    stat  = 0
  end function
!======================================================================
!.. select the reference velocity for the current depth step
!.. Lloyd's method will be used
!----------------------------------------------------------------------
  integer function bwi_ref_oper(vmod_slice, qmod_slice, vref_slice, qref_slice, zref, iregion) result(stat)
    real, dimension(:,:) :: vmod_slice
    real, dimension(:)   :: vref_slice
    real, dimension(:,:) :: qmod_slice
    real, dimension(:)   :: qref_slice

    real, allocatable :: attr(:,:),slice(:,:),junk(:,:)
    integer,allocatable :: iregion(:)
    integer ::iref,n(3), zref, ix, iy
   stat = 1

   call init_lloyd();

   n(1)=2; n(2)=nx; n(3)=ny;

   allocate(junk(n(1),n(2)*n(3)),slice(n(2)*n(3),n(1)), attr(nrf,n(1)))
 
  ! nstart=(/1,3/)
   if(product(nstart) > nrf) &
   call seperr("product(nstart) > nrf")
 do iy=1,ny   
 do ix=1,nx
      junk(1,(iy-1)*nx+ix)=vmod_slice(ix,iy)
      junk(2,(iy-1)*nx+ix)=qmod_slice(ix,iy)*qscale
   end do
   end do
   slice=transpose(junk)
!write(0,*) "slice", slice, size(slice)
   iref= lloyd_go(slice, attr, nrf, iregion,nstart)
     if(iref < nrf) then
       attr(iref+1:,:)=-1
     end if
   vref_slice=attr(:,1)
   qref_slice=attr(:,2)/qscale

   zref=iref
  !write(0,*) "zref", zref
   stat = 0

  end function


end module
