module fourier

implicit none
contains

!===========================================
subroutine ft1axis( adj, sign1, n1,n2, cx)
integer i2, adj, n1,n2
complex cx(n1,n2)
real sign1
do i2= 1, n2
  call fth( adj, sign1, 1,n1, cx(1,i2))
end do
end subroutine
!===========================================
subroutine ft2axis( adj, sign2, n1,n2, cx)
integer i1, adj, n1,n2
complex cx(n1,n2)
real sign2
do i1= 1, n1
  call fth( adj, sign2, n1,n2, cx(i1,1))
end do
end subroutine
!===========================================
subroutine fth( adj,sign, m1, n12, cx)
integer i, adj, m1, n12
real sign
complex cx(m1,n12)
complex temp(n12)
do i= 1, n12
  temp(i) = cx(1,i)
end do
if ( adj .eq. 0) then
  do i= 2, n12, 2
    temp(i) = -temp(i)
  end do
  call ftu( sign, n12, temp)
else
  call ftu( -sign, n12, temp)
  do i= 2, n12, 2
    temp(i) = -temp(i)
  end do
end if
do i= 1, n12
  cx(1,i) = temp(i)
end do
end subroutine
!===========================================
subroutine ftu( signi, nx, cx )
! complex fourier transform with unitary scaling
!
! 1 nx signi*2*pi*i*(j-1)*(k-1)/nx
! cx(k) = ---- * sum cx(j) * e
! sqrt(nx) j=1 for k=1,2,...,nx=2**integer
!
integer nx, i, j, k, m, istep
real signi, scale, arg
complex cx(nx), cmplx, cw, cdel, ct
call pad2(nx)

scale = 1. / sqrt( 1.*nx)
do i= 1, nx
  cx(i) = cx(i) * scale
end do
j = 1
k = 1
do i= 1, nx
  if (i<=j) then
    ct = cx(j)
    cx(j) = cx(i)
    cx(i) = ct
  end if
  m = nx/2
  do while (j>m .and. m>1)
    j = j-m
    m = m/2
  end do
! "&&" means .AND.
  j = j+m
end do
do
  istep = 2*k
  cw = 1.
  arg = signi*3.14159265/k
  cdel = cmplx( cos(arg), sin(arg))
  do m= 1, k
    do i= m, nx, istep
      ct=cw*cx(i+k)
      cx(i+k)=cx(i)-ct
      cx(i)=cx(i)+ct
    end do
    cw = cw * cdel
  end do
  k = istep
  if (k>=nx) then
    exit
  end if
end do
end subroutine
!============================================
subroutine pad2(nx)
integer	:: nx,ni

ni=1
do while (nx > ni)
   ni=ni*2
end do
nx=ni

end subroutine
!===========================================
end module
