module toeplitz_mod
  ! Module to solve inversions involving Toeplitz matrices
  !
  ! Jason Chang
  ! February 2016

  !=========================================
  ! includes
  !=========================================
  implicit none


  !=========================================
  ! declare module variables
  !=========================================

  contains

  !=========================================
  ! initialize toeplitz
  !=========================================
  subroutine toeplitz_init()
  end subroutine


  !=========================================
  ! close toeplitz
  !=========================================
  subroutine toeplitz_close()
  end subroutine


  !=========================================
  ! solve symmetric toeplitz
  !=========================================
  subroutine toeplitz(trow,ocol,filter)
    ! declare variables
    integer                       :: ii,jj
    real                          :: v,e,c,w,bot
    real,dimension(:)             :: filter,trow,ocol
    real,allocatable,dimension(:) :: a

    ! check if main diagonal is zero
    if (trow(1).eq.0.) then
      return
    end if

    allocate(a(size(ocol))); a=0.

    ! set values
    a(1) = 1.
    v = trow(1)
    filter(1) = ocol(1)/trow(1)

    ! recursively build filter
    do jj=2,size(ocol)
      ! solve Ta = v
      a(jj) = 0.
      filter(jj) = 0.

      e = 0.
      do ii=1,jj-1
        e = e+a(ii)*trow(jj-ii+1)
      end do

      c = e/v
      v = v-(c*e)

      do ii=1,(jj+1)/2
        bot = a(jj-ii+1)-(c*a(ii))
        a(ii) = a(ii)-(c*a(jj-ii+1))
        a(jj-ii+1) = bot
      end do

      w = 0.
      do ii=1,jj-1
        w = w+filter(ii)*trow(jj-ii+1)
      end do

      c = (w-ocol(jj))/v
      do ii=1,jj
        filter(ii) = filter(ii)-(c*a(jj-ii+1))
      end do
    end do

    deallocate(a)
    
  end subroutine

end module
